import { View, Canvas } from '@tarojs/components'
import Taro, { useDidHide, useDidShow } from '@tarojs/taro'
import { useCallback, useEffect, useMemo, useRef, useState } from 'react'
import { throttle } from '@/utils'
import newShake from '@/utils/shake'
import S from '@/utils/shape-shifter'

import './style.less'

const { onShake } = newShake()

const ShakeCount = (props) => {
  const { className, onTimes } = props
  const classNames = useMemo(() => [className, 'shake-count'].filter(Boolean).join(' '), [className])

  // const [times, setTimes] = useState(0)
  const timesRef = useRef(0)
  // timesRef.current = times

  const canvasId = useMemo(() => `f-${Math.random().toString(36).substring(2)}`, [])
  const canvasRef = useRef()
  const canvasShapeRef = useRef()

  // eslint-disable-next-line react-hooks/exhaustive-deps
  const handleShake = useCallback(throttle((times) => {
    S.performAction(times)

    if (onTimes) {
      onTimes(times)
    }
  }, 500, { trailing: true, immediate: true }), [])

  const initS = useCallback(() => {
    if (canvasRef.current && canvasShapeRef.current) {

      S.init({
        canvas: canvasRef.current,
        shape: canvasShapeRef.current,
        width: canvasRef.current.width,
        height: canvasRef.current.height
      })

      S.performAction(0)
    }
  }, [])

  useDidShow(() => {
    Taro.startAccelerometer()

    Taro.onAccelerometerChange((e) => {
      onShake(e, () => {
        timesRef.current += 1
        console.log('---times: ', timesRef.current)
        handleShake(timesRef.current)
        // S.performAction(timesRef.current)
      })
    })
  })

  useDidHide(() => {
    Taro.startAccelerometer()
  })

  useEffect(() => {
    Taro.nextTick(() => {
      Taro.createSelectorQuery()
        .select(`#${canvasId}`)
        .fields({ node: true, size: true })
        .exec((res) => {
          const canvas = res[0].node
          const ctx = canvas.getContext('2d')

          const dpr = Taro.getSystemInfoSync().pixelRatio
          const width = res[0].width * dpr
          const height = res[0].height * dpr
          canvas.width = width
          canvas.height = height
          ctx.scale(dpr, dpr)
          
          canvasRef.current = canvas
          initS()
        })

      Taro.createSelectorQuery()
        .select(`#${canvasId}-canvas-shape`)
        .fields({ node: true, size: true })
        .exec((res) => {
          const canvas = res[0].node
          const ctx = canvas.getContext('2d')

          const dpr = Taro.getSystemInfoSync().pixelRatio
          const width = res[0].width * dpr
          const height = res[0].height * dpr
          canvas.width = width
          canvas.height = height
          ctx.scale(dpr, dpr)
          
          canvasShapeRef.current = canvas
          initS()
        })
    })
  }, [canvasId, initS])

  return (
    <View className={classNames}>
      <Canvas id={`${canvasId}-canvas-shape`} className='canvas-shape' type='2d'></Canvas>
      <Canvas id={canvasId} type='2d'></Canvas>
    </View>
  )
}

export default ShakeCount;
